;	########################
;	#                      #
;	#  BOUNCING SPIKE BALL #
;	#                      #
;	#  by WhiteYoshiEgg    #
;	#                      #
;	########################



;	This is basically a spiked ball
;	that hops up and down while moving
;	in Mario's direction.
;	There's no way to kill it, you'll just
;	have to avoid it. It can be quite challenging
;	when more than one is on-screen. 



print "INIT ",pc
		RTL         

                    
print "MAIN ",pc
		PHB
		PHK
		PLB
		JSR SpriteCode
		PLB
		RTL


;##############################################################


	!JumpingSpeed   =  $B0		;    Jumping Speed ($FF=low, $80=high)

	SpeedTable:	db $10,$F0	;    X speeds (right, left)

	Tilemap:	db $00,$02	;    Sprite Tiles

SpriteCode:

		JSR SubGFX		;    Draw Graphics

		LDA $9D
		BNE Return

		LDA $1588,x       	; \  Only do the following
		AND #$04		;  | when the sprite is
		BEQ DontJump		; /  touching the ground
		
		LDA #!JumpingSpeed	; \  Jump
		STA $AA,x		; /
		
		PHY			; \
		JSR SUB_HORZ_POS	;  | Go right or left
		LDA SpeedTable,y	;  | depending on which side
		PLY			;  | of the sprite Mario stands
		STA $B6,x		; /

DontJump:
		
		JSL $01802A		; Update position + add gravity

		JSL $01A7DC		; \
		BCC Return		;  | Kill Mario if touched
		JSL $00F5B7		; /

Return:
		RTS










SubGFX:             JSR GET_DRAW_INFO       ; after: Y = index to sprite OAM ($300)
                                            ;      $00 = sprite x position relative to screen boarder 
                                            ;      $01 = sprite y position relative to screen boarder  
                    
                    LDA $157C,x             ; $02 = sprite direction
                    STA $02

                    ; if you wish to draw more than one tile, each step between the lines must be repeated
                    ;*************************************************************************************  

                    LDA $00                 ; set x position of the tile
                    STA $0300,y

                    LDA $01                 ; set y position of the tile
                    STA $0301,y

                    PHX
		    LDA $14
		    LSR
		    LSR
		    LSR
		    AND #$01
		    TAX
                    LDA Tilemap,x
		    PLX
                    STA $0302,y

                    LDA $15F6,x             ; get sprite palette info
                    PHX
                    LDX $02                 ; flip the tile if the sprite direction is 0
                    BNE NO_FLIP_1
                    ORA #$40
NO_FLIP_1:           PLX
                    ORA $64                 ; add in the priority bits from the level settings
                    STA $0303,y             ; set properties

                    INY                     ; get the index to the next slot of the OAM
                    INY                     ; (this is needed if you wish to draw another tile)
                    INY
                    INY
                    
                    ;*************************************************************************************                  

                    LDY #$02                ; #$02 means the tiles are 16x16
                    LDA #$00                ; This means we drew one tile
                    JSL $01B7B3
                    RTS



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; ROUTINES FROM THE LIBRARY ARE PASTED BELOW
; You should never have to modify this code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;       Y = index to sprite OAM ($300)
;       $00 = sprite x position relative to screen boarder
;       $01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1:              db $0C,$1C
SPR_T2:              db $01,$02

GET_DRAW_INFO:       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X:         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.w #$0040            ;  |
                    CMP.w #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP:      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC SPR_T1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA SPR_T2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y:         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID:             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /




SUB_HORZ_POS:				LDY #$00
					LDA $94	
					SEC                 
					SBC $E4,x				
					STA $0F					
					LDA $95					
					SBC $14E0,x				
					BPL SPR_L16            
					INY                     
SPR_L16:				RTS                     
